import os
from pathlib import Path
import sqlite3


STT_SERVICES = {
        'amazon': {
                'cost_per_minute': .024,
                'free': '60_minutes_per_month_for_the_first_12_months',
            },
        }


AWS_BUCKET_NAME_FMTR_MEDIA = 'tatt-media-{}'
AWS_BUCKET_NAME_FMTR_TRANSCRIPT = 'tatt-transcript-{}'
AWS_CREDENTIALS_FILEPATH = (
    os.getenv('AWS_CREDENTIALS_FILEPATH') 
    or Path.home() / '.aws/credentials'
)
AWS_REGION = 'us-east-1'

SERVICE_CLASS_NAME = 'transcribe'



import json
from pprint import pprint
import sqlite3
import sys

import click

from tatt import config, exceptions, helpers, vendors


@click.group()
def cli():
    pass


@cli.command()
@click.option('-f', '--file', is_flag=True, help='save to file')
@click.argument('name')
def get(name, file):
    """Downloads and/or saves completed transcript."""
    job = helpers.get_transcription_jobs_dict().get(name)
    if not job:
        raise click.ClickException(f'no such transcript {name}')
    if job['status'].lower() != 'completed':
        raise click.ClickException(f'transcript status is {job["status"]}')
    service = helpers.get_service(job['service_name'])
    if not file:
        pprint(service.retrieve_transcript(name))
    else:
        with open(f'{name}.json', 'w') as fout:
            fout.write(json.dumps(service.retrieve_transcript(name)))
            print(f'Okay, downloaded {name}.json')



@cli.command()
@click.option('-n', '--name', type=str, help="transcription job name")
@click.option('--service', type=str, help="STT service name")
@click.option('--status', type=str, help="completed | failed | in_progress")
def list(name, service, status):
    """Lists available STT services."""
    if service is not None and service not in config.STT_SERVICES:
        raise click.ClickException(f'no such service: {service}')

    try:
        all_jobs = helpers.get_transcription_jobs(service, name, status)
    except exceptions.ConfigError as e:
        raise click.ClickException(str(e))
    else:
        if not all_jobs:
            raise click.ClickException('no transcripts currently!') 

        helpers.print_transcription_jobs(all_jobs)


@cli.command()
@click.option('-f', '--free-only', is_flag=True, help='only free services')
def services(free_only):
    """Lists available speech-to-text services."""
    click.echo(helpers.make_string_all_services(free_only))


@cli.command()
@click.option('-d', '--dry-run', is_flag=True, help=(
  'Do a dry run without actually submitting the media file for transcription'))
@click.argument('media_filepath', type=str)
@click.argument('service_name', type=str)
def this(dry_run, media_filepath, service_name):
    """Sends a media file to be transcribed."""
    if service_name not in config.STT_SERVICES:
        print()
        raise click.ClickException(
            f'No such service! {print_all_services(print_=False)}')

    service = helpers.get_service(service_name)
    s = service(media_filepath)

    if dry_run:
        print('If this weren\'t a dry run, I would transcribe '
             f'{media_filepath} using {service_name}')
        pprint(vars(s))
    else:
        print(
          f'Okay, transcribing {media_filepath} using {service_name}...')

        try:
            job_num = s.transcribe()
        except exceptions.AlreadyExistsError as e:
            raise click.ClickException(str(e))
        print(f'Okay, job {job_num} is being transcribed.  Use "get" '
               'command to download it.')



import json
import os
from pathlib import PurePath
from subprocess import check_output
import uuid

import boto3

from tatt import config
from tatt import exceptions

NAME = 'amazon'
BUCKET_NAME_MEDIA = config.AWS_BUCKET_NAME_FMTR_MEDIA.format(NAME)
BUCKET_NAME_TRANSCRIPT = config.AWS_BUCKET_NAME_FMTR_TRANSCRIPT.format(NAME)
tr = boto3.client('transcribe')
s3 = boto3.resource('s3')


class transcribe:

    bucket_names = {'media': BUCKET_NAME_MEDIA,
                    'transcript': BUCKET_NAME_TRANSCRIPT}
    service_name = 'amazon'

    def __init__(self, filepath):
        self._setup()
        self.filepath = PurePath(filepath)
        self.basename = str(os.path.basename(self.filepath))
        self.media_file_uri = (
                f"https://s3-{config.AWS_REGION}.amazonaws.com/"
                f"{self.bucket_names['media']}/{self.basename}")

    @staticmethod
    def _setup():
        if not check_for_credentials():
            raise exceptions.ConfigError('please run "aws configure" first')
        self = transcribe
        for bucket_name in self.bucket_names.values():
            if not self.check_for_bucket(bucket_name):
                self.make_bucket(bucket_name)

    @staticmethod
    def check_for_bucket(bucket_name):
        return bool(s3.Bucket(bucket_name).creation_date)

    @staticmethod
    def make_bucket(bucket_name):
        s3.create_bucket(Bucket=bucket_name)

    def transcribe(self):
        self._upload_file()
        try:
            return self._request_transcription()
        except tr.exceptions.ConflictException:
            raise exceptions.AlreadyExistsError(
                f'{self.basename} already exists on {self.service_name}')

    def _upload_file(self):
        s3.Bucket(self.bucket_names['media']).upload_file(
                str(self.filepath),
                self.basename)

    def _request_transcription(self, language_code='en-US'):
        job_name = self.basename
        tr.start_transcription_job(
                TranscriptionJobName=job_name,
                LanguageCode=language_code,
                MediaFormat=self.basename.split('.')[-1].lower(),
                Media={
                    'MediaFileUri': self.media_file_uri
                    },
                OutputBucketName=self.bucket_names['transcript']
                )
        return job_name

    @staticmethod
    def get_completed_jobs(job_name_query=None):
        return transcribe.get_transcription_jobs(
                status='completed',
                job_name_query=job_name_query)

    @staticmethod
    def get_pending_jobs(job_name_query=None):
        return transcribe.get_transcription_jobs(
                status='in_progress',
                job_name_query=job_name_query)

    @staticmethod
    def get_all_jobs(job_name_query=None):
        return transcribe.get_transcription_jobs(job_name_query)

    @staticmethod
    def get_transcription_jobs(status=None, job_name_query=None):
        kwargs = {'MaxResults': 100}
        if status is not None:
            kwargs['Status'] = status.upper()
        if job_name_query is not None:
            kwargs['JobNameContains'] = job_name_query
        jobs_data = tr.list_transcription_jobs(**kwargs)
        jobs = homogenize_transcription_job_data(jobs_data['TranscriptionJobSummaries'])
        while jobs_data.get('NextToken'):
            jobs_data = tr.list_transcription_jobs(NextToken=jobs_data['NextToken'])
            jobs += homogenize_transcription_job_data(
                        jobs_data['TranscriptionJobSummaries'])
        return jobs

    @staticmethod
    def retrieve_transcript(transcription_job_name):
        job = tr.get_transcription_job(
            TranscriptionJobName=transcription_job_name
        )['TranscriptionJob']

        if not job['TranscriptionJobStatus'] == 'COMPLETED':
            return

        transcript_file_uri = job['Transcript']['TranscriptFileUri']
        transcript_path = transcript_file_uri.split("amazonaws.com/", 1)[1]

        transcript_bucket = transcript_path.split('/', 1)[0]
        transcript_key = transcript_path.split('/', 1)[1]

        s3_object = s3.Object(transcript_bucket, transcript_key).get()
        transcript_json = s3_object['Body'].read().decode('utf-8')
        return json.loads(transcript_json)




def homogenize_transcription_job_data(transcription_job_data):
    return [{
                'created': jd['CreationTime'],
                'name': jd['TranscriptionJobName'],
                'status': jd['TranscriptionJobStatus']
            }
            for jd in transcription_job_data]


def check_for_credentials():
    return config.AWS_CREDENTIALS_FILEPATH.exists()


def shell_call(command):
    return check_output(command, shell=True)



from setuptools import setup, find_packages


with open('README.md') as file:
    long_description = file.read()

setup(
    name="tatt",
    version="0.16",
    py_modules=['tatt'],
    url='https://github.com/zevaverbach/tatt',
    install_requires=[
        'Click',
        'awscli',
        'boto3',
        'requests',
        ],
    include_package_data=True,
    packages=find_packages(),
    description=('Tatt creates a uniform API for multiple speech-to-text '
                 '(STT) services.'),
    long_description_content_type='text/markdown',
    long_description=long_description,
    entry_points='''
        [console_scripts]
        transcribe=tatt.transcribe:cli
    ''',
        )



from tatt import config
from tatt import vendors


def make_string_all_services(free_only=False):
    all_services_string = (
          '\nHere are all the available '
         f'{"free " if free_only else ""}speech-to-text services:'
          '\n\n'
          '\n'.join(['{}{}{}{}'.format('\t', service_name, '\t\t',

                       f'({info["free"].replace("_", " ")})' 
                       if isinstance(info["free"], str) else ""
                       
                                      )

                     for service_name, info in
                     config.STT_SERVICES.items()]) + '\n'
    )
    return all_services_string


def get_service(service_name):
    return getattr(getattr(vendors, service_name), config.SERVICE_CLASS_NAME)


def print_transcription_jobs(jobs):
    max_job_name_length = max(len(job['name']) 
                                  for job_list in jobs.values()
                                  for job in job_list)
    max_service_name_length = max(len(provider_name) for provider_name in jobs)
    print()
    print('Service',
          'Job Name', 
          (max_job_name_length - len('Job Name')) * ' ',
          '  Status')
    print('-------',
          '--------',
          (max_job_name_length - len('Job Name')) * ' ', 
          '  ------')

    for provider_name, job_list in jobs.items():
        for job in job_list:
            num_spaces_between = max_job_name_length - len(job['name'])
            print(provider_name, job['name'], ' ' * num_spaces_between,
                    job['status'], sep='  ')
    print()


def get_transcription_jobs(service_name=None, name=None, status=None):
    all_jobs = {}
    for stt_name, data in config.STT_SERVICES.items():
        if service_name is None or service_name == stt_name:
            service = get_service(stt_name)
            service._setup() # check for AWS credentials and create buckets
            jobs = service.get_transcription_jobs(job_name_query=name,
                                                  status=status)
            if jobs:
                all_jobs[stt_name] = jobs
    return all_jobs


def get_transcription_jobs_dict():
    jobs = get_transcription_jobs()
    return {
            job['name']: {
                'service_name': service_name,
                'status': job['status']
                }
            for service_name, job_list in jobs.items()
            for job in job_list
            }
